//
//  CharacterView.m
//  ColorsWar
//
/*
 * HAZ™ Dev Group: http://imhaz.com
 *
 * Copyright (c) 2007-2011 HAZ™ Inc.
 *
 * Written by Hercy Chang 2011-12-08
 *
 */

#import "CharacterView.h"
#import "CWColor.h"

#define pifactor (float)M_PI/180

@implementation CharacterView
@synthesize pointsOfShape;
@synthesize theColor,backcolor;
@synthesize chosedPoint;
@synthesize drawBoundary;

void advancedLine(CGPoint sp,CGPoint ep,CGRect startc,CGRect endc,CGRect edgecolor,float width)
{
	float anglestoe = AngleOfTwoPoints(sp, ep);
	float radfactor = (float)M_PI/180;
	
	GLfloat vertices[12];
	GLfloat colors[24];
	
	float dx1 = width/2*sinf((anglestoe+90)*radfactor);
	float dy1 = width/2*cosf((anglestoe+90)*radfactor);
	float dx2 = width/2*sinf((anglestoe+270)*radfactor);
	float dy2 = width/2*cosf((anglestoe+270)*radfactor);
	
	vertices[0] = sp.x+dx1;
	vertices[1] = sp.y+dy1;
	vertices[2] = ep.x+dx1;
	vertices[3] = ep.y+dy1;
	vertices[4] = sp.x;
	vertices[5] = sp.y;
	vertices[6] = ep.x;
	vertices[7] = ep.y;
	vertices[8] = sp.x+dx2;
	vertices[9] = sp.y+dy2;
	vertices[10] = ep.x+dx2;
	vertices[11] = ep.y+dy2;
	
	colors[0] = colors[4] = colors[16] = colors[20] = edgecolor.origin.x;
	colors[1] = colors[5] = colors[17] = colors[21] = edgecolor.origin.y;
	colors[2] = colors[6] = colors[18] = colors[22] = edgecolor.size.width;
	colors[3] = colors[7] = colors[19] = colors[23] = edgecolor.size.height;
	
	colors[8] = startc.origin.x;
	colors[9] = startc.origin.y;
	colors[10] = startc.size.width;
	colors[11] = startc.size.height;
	
	colors[12] = endc.origin.x;
	colors[13] = endc.origin.y;
	colors[14] = endc.size.width;
	colors[15] = endc.size.height;
	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 6);
	
	// restore default state
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);		
	
}

void drawShape( NSArray* points, CGRect centercolor, CGRect edgecolor,float width )
{
	int num = [points count];
	for(int i=0;i<num;i++){
		CGPoint p1,p2;
		if(i==num-1){
			p1 = [[points lastObject] CGPointValue];
			p2 = [[points objectAtIndex:0] CGPointValue];
		}
		else {
			p1 = [[points objectAtIndex:i] CGPointValue];
		    p2 = [[points objectAtIndex:i+1] CGPointValue];	
		}
		advancedLine(p1, p2, centercolor, centercolor, edgecolor, width);
	}	
}
-(void)draw{
	if(chosedPoint>=0){
		CGPoint chop = [[pointsOfShape objectAtIndex:chosedPoint] CGPointValue];
		float angle = CC_DEGREES_TO_RADIANS(45);
		glColor4f(1,1,1,0);
		ccDrawPoint(chop);
		ccDrawCircle(chop, 5, angle, 4, NO);
	}
	if(drawBoundary){
		advancedLine(ccp(-50,0), ccp(50,0), WhiteWithAlpha(0), WhiteWithAlpha(0),BlackWithAlpha(0), 0.5);
		advancedLine(ccp(0,50), ccp(0,-50), WhiteWithAlpha(0), WhiteWithAlpha(0),BlackWithAlpha(0), 0.5);
		
	}
	float adaptedw = 4.5/self.scale;
	if(adaptedw<2)adaptedw=2;
	drawShape(pointsOfShape, theColor, BlackWithAlpha(0),adaptedw);

}

-(void)dealloc{
	[super dealloc];
	//[pointsOfShape removeAllObjects];
}

-(id)init{
	if((self=[super init])){
		pointsOfShape = [[NSMutableArray alloc] initWithObjects:[NSValue valueWithCGPoint:ccp(0,27)],
						 [NSValue valueWithCGPoint:ccp(-15,-15)],
						 [NSValue valueWithCGPoint:ccp(15,-15)],nil];
		theColor = WhiteWithAlpha(1);
		backcolor = WhiteWithAlpha(0);
		chosedPoint = 0;
		drawBoundary = NO;
	}
	return self;
}

-(int)thePointNearestTo:(CGPoint)p{
	CGPoint p0 = [[pointsOfShape objectAtIndex:0] CGPointValue];
	float dis = ccpDistance(p, p0);
	int i=0;
	int j=0;
	for(NSValue *pv in pointsOfShape){
		CGPoint pn = [pv CGPointValue];
		float disn = ccpDistance(p, pn);
		if(disn<dis){
			dis = disn;
			i=j;
		}
		j++;
	}
	return i;
}

-(void)changeCurrentPointTo:(CGPoint)p{
	float x,y;
	x = p.x;
	y = p.y;
	
	if(x>50)
		x=50;
	else if(x<-50)
		x=-50;
	
	if(y>50)
		y=50;
	else if(y<-50)
		y=-50;
	[pointsOfShape replaceObjectAtIndex:chosedPoint withObject:[NSValue valueWithCGPoint:ccp(x,y)]];
}

-(IBAction)xadd:(id)sender{
	CGPoint p = [[pointsOfShape objectAtIndex:chosedPoint] CGPointValue];
	if(p.x>=50)return;
	p.x = p.x+0.1;
	[pointsOfShape replaceObjectAtIndex:chosedPoint withObject:[NSValue valueWithCGPoint:p]];
}
-(IBAction)xsub:(id)sender{
	CGPoint p = [[pointsOfShape objectAtIndex:chosedPoint] CGPointValue];
	if(p.x<=-50)return;
	p.x = p.x-0.1;
	[pointsOfShape replaceObjectAtIndex:chosedPoint withObject:[NSValue valueWithCGPoint:p]];
	
}
-(IBAction)yadd:(id)sender{
	CGPoint p = [[pointsOfShape objectAtIndex:chosedPoint] CGPointValue];
	if(p.y>=50)return;
	p.y = p.y+0.1;
	[pointsOfShape replaceObjectAtIndex:chosedPoint withObject:[NSValue valueWithCGPoint:p]];
	
}
-(IBAction)ysub:(id)sender{
	CGPoint p = [[pointsOfShape objectAtIndex:chosedPoint] CGPointValue];
	if(p.y<=-50)return;
	p.y = p.y-0.1;
	[pointsOfShape replaceObjectAtIndex:chosedPoint withObject:[NSValue valueWithCGPoint:p]];
	
}

-(IBAction)rotate:(id)sender{
	NSMutableArray *rotated = [NSMutableArray arrayWithObjects:nil];
	for(NSValue *pv in pointsOfShape){
		CGPoint p = [pv CGPointValue];
		CGPoint tp = ccp(p.y,-p.x);
		[rotated addObject:[NSValue valueWithCGPoint:tp]];	
	}
	[pointsOfShape removeAllObjects];
	[pointsOfShape addObjectsFromArray:rotated];
}

-(IBAction)addnew:(id)sender{
	int total = [pointsOfShape count];
	if(total>=100)
		return;
	CGPoint n = ccp((CCRANDOM_0_1()-0.5)*100,(CCRANDOM_0_1()-0.5)*100);
	
	NSMutableArray *oldpoints = [NSMutableArray arrayWithArray:pointsOfShape];
	[pointsOfShape removeAllObjects];
	int i=0;
	for(;i<chosedPoint+1;i++)
		[pointsOfShape addObject:[oldpoints objectAtIndex:i]];
	
	[pointsOfShape addObject:[NSValue valueWithCGPoint:n]];
	
	for(;i<total;i++)
		[pointsOfShape addObject:[oldpoints objectAtIndex:i]];
	chosedPoint++;
	
}
-(IBAction)deletes:(id)sender{
	if([pointsOfShape count]>3)[pointsOfShape removeObjectAtIndex:chosedPoint];
	if(chosedPoint>0)chosedPoint = chosedPoint-1;
}


@end
