//
//  DataOfTheAncients.m
//  ColorsWar
//
/*
 * HAZ™ Dev Group: http://imhaz.com
 *
 * Copyright (c) 2007-2011 HAZ™ Inc.
 *
 * Written by Hercy Chang 2011-12-08
 *
 */

#define kFilename @"cwDataArchiver.plist"
#define kSavedFile @"cwData"

#define kShipShape @"shapeData"
#define kShipColor @"colorData"
#define kBase @"baseData"
#define kDPoints @"dPointsData"

#import "DataOfTheAncients.h"
#import "ApostleBasis.h"
#import "CWColor.h"

@interface DataOfTheAncients (Private)

-(NSString*)dataFilePath;
@end


@implementation DataOfTheAncients

@synthesize theship;
@synthesize thebase;
@synthesize drawnPoints;

static DataOfTheAncients* instanceOfDOTA;

- (NSString *)dataFilePath
{
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	return [documentsDirectory stringByAppendingPathComponent:kFilename];
}

+(DataOfTheAncients*) sharedDOTA;
{
	NSAssert(instanceOfDOTA != nil, @"Dota instance not yet initialized!");
	return instanceOfDOTA;
}

-(void)dealloc{
	[super dealloc];
	[thebase release];
	[drawnPoints release];
}

-(void)saveTheData{
	NSMutableData *data = [[NSMutableData alloc] init];
	NSKeyedArchiver *archiver = [[NSKeyedArchiver alloc] initForWritingWithMutableData:data];
	
	NSDictionary *savdic = [NSDictionary dictionaryWithObjectsAndKeys:
							thebase,kBase,
							theship.theShape,kShipShape,
							[NSValue valueWithCGRect:theship.selfcolor],kShipColor,
							drawnPoints,kDPoints,
							nil];
	
	[archiver encodeObject:savdic forKey:kSavedFile];
	[archiver finishEncoding];
	[data writeToFile:[self dataFilePath] atomically:YES];
	[archiver release];
	[data release];
}

-(void)loadFromPlist{
	theship = [ApostleBasis node];
	drawnPoints = [[NSMutableArray alloc] init];
	
	NSString *filePath = [self dataFilePath];
	if ([[NSFileManager defaultManager] fileExistsAtPath:filePath])
	{
		NSLog(@"Load!");
		NSData *cwdata = [[NSMutableData alloc] initWithContentsOfFile:filePath];
		NSKeyedUnarchiver *unarchiver = [[NSKeyedUnarchiver alloc] initForReadingWithData:cwdata];
		NSDictionary *loadic = [unarchiver decodeObjectForKey:kSavedFile];
		
		[unarchiver finishDecoding];
		thebase = [[NSMutableDictionary alloc] initWithDictionary:[loadic objectForKey:kBase]];
		[theship.theShape removeAllObjects];
		[theship.theShape addObjectsFromArray:[loadic objectForKey:kShipShape]];
		theship.selfcolor = [[loadic objectForKey:kShipColor] CGRectValue];
		[drawnPoints addObjectsFromArray:[loadic objectForKey:kDPoints]];
		
		[unarchiver release];
		[cwdata release];
	}
	else {
		thebase = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
				   [NSNumber numberWithInt:0],kTKills,
				   [NSNumber numberWithInt:0],kTDeathes,
				   [NSNumber numberWithInt:0],kTTime,
				   [NSNumber numberWithInt:0],kBTime,
				   nil];		
	}
}

-(id)init{
	if((self=[super init])){
		instanceOfDOTA = self;
		
		[self loadFromPlist];
	}
	return self;
}

-(void)addK:(int)k De:(int)de T:(int)t{
	int ok = [[thebase objectForKey:kTKills] intValue];
	int ode = [[thebase objectForKey:kTDeathes] intValue];
	int ot = [[thebase objectForKey:kTTime] intValue];
	int bt = [[thebase objectForKey:kBTime] intValue];
	
	[thebase setObject:[NSNumber numberWithInt:ok+k] forKey:kTKills];
	[thebase setObject:[NSNumber numberWithInt:ode+de] forKey:kTDeathes];
	[thebase setObject:[NSNumber numberWithInt:ot+t] forKey:kTTime];
	
	if(bt==0 || (t<bt && t>0))
		[thebase setObject:[NSNumber numberWithInt:t] forKey:kBTime];
}

-(void)addDrawnPointsFrom:(NSMutableArray*)dp{
	int total = [drawnPoints count];
	if(total>=100)
		[drawnPoints removeObjectAtIndex:0];
	[drawnPoints addObject:dp];
}

@end
