/*
 * HAZ™ Dev Group: http://imhaz.com
 *
 * Copyright (c) 2007-2011 HAZ™ Inc.
 *
 * Written by Hercy Chang 2011-12-08
 *
 */

#import "GameController.h"
#import "GameView.h"
#import "DataLayer.h"
#import "DataOfTheAncients.h"
#import "CWColor.h"
#import "ApostleBasis.h"
#import "AnimationLayer.h"

@interface GameController (Private)

-(void)initWithExploreMode;
-(void)initWithShootMode;
-(void)initWithVSMode;
-(void)addGameView;
-(void)addActionController;
-(void)addGameUI;

@end


@implementation GameController

+(id) scene
{
	CCScene *scene = [CCScene node];
	
	GameController *layer = [GameController node];
	[scene addChild: layer];
		
	return scene;
}

-(void)initWithExploreMode{
	[self addGameView];
	[self addActionController];
	[self addGameUI];
	[self schedule:@selector(exploreGameModeupdate) interval:1.0/30];
}

-(void)initWithShootMode{
	[self addGameView];
	[self addActionController];
	[self addGameUI];
	[self schedule:@selector(shootModeupdate) interval:1.0/30];
}

-(void)initWithVSMode{
	[self addGameView];
	[self addActionController];
	[self addGameUI];
	[self schedule:@selector(vsModeupdate) interval:1.0/30];
}

-(void)addGameUI{
	theUIView = [GameUI node];
	theUIView.colorToDrawUI = [DataLayer sharedDataLayer].mycolorfortest;
	[self addChild:theUIView];
}
-(void)addActionController{
	actionController = [MoveAndScale node];
	actionController.colortodraw = [DataLayer sharedDataLayer].mycolorfortest;
	[self addChild:actionController];
}
-(void)addGameView{
	GameView *gview = [GameView node];
	[self addChild:gview];
}
-(void)dealloc{
	[super dealloc];
}
-(id)init{
	if((self=[super init])){
		i=k=d=t=w=firecounter=0;
		whetherAlive = YES;
		screenSize = [[CCDirector sharedDirector] winSize];
		DataLayer *dl = [DataLayer sharedDataLayer];
		switch (dl.gameMode) {
			case GameModeExplore:{
				[self initWithExploreMode];
				break;}
			case GameModeShoot:
				[self initWithShootMode];
				break;
			case GameModeFind:
				[self addActionController];
				[self addGameView];
				[self schedule:@selector(achieveModeupdate) interval:1.0/30];
				break;
			case GameModeChallenge:
				[self initWithVSMode];
				break;

			default:
				break;
		}
	}
	return self;
}

-(void)exploreGameModeupdate{
	if(i>=300)i=0;
	i++;
		
	GameView *gview = [GameView sharedGameView];
	DataLayer *dl = [DataLayer sharedDataLayer];
	ApostleBasis *m = gview.mc;
	NSMutableArray *es = gview.allEnemies;
			
	if(m.visible){
		m.position = gview.centerPosition;
		
		if(actionController.isMoving){
			[m turnto:actionController.angle atSpeed:5];
			if(!m.haveShield){
				[gview moveCenterAt:actionController.speed WithRotation:actionController.angle];	
				[[AnimationLayer sharedAnimationLayer] showMoveEffectAt:m.position WithRotation:actionController.angle Speed:actionController.speed];
				
			}
		}
			
		else [[AnimationLayer sharedAnimationLayer] stopMoveEffect];
		
	}
	else {
		[[AnimationLayer sharedAnimationLayer] stopMoveEffect];
		[gview moveCenterAt:actionController.speed WithRotation:actionController.angle];
		if(actionController.shoot){
			m.currentEnergy = m.maxEnergy;
			m.position = gview.centerPosition;
			[m becomeActiveExplore];
		}
	}
	
	actionController.maxZoomfactor = m.visualDistance*6/(screenSize.height+screenSize.width);
	if(actionController.isScaling)
		[gview zoomGameViewWith:actionController.zoomfactor];
	
	theUIView.currentE = m.currentEnergy;
	theUIView.maxE = m.maxEnergy;
	if(whetherAlive==YES && m.visible==NO){
		d++;
		[[AnimationLayer sharedAnimationLayer] addMessage:@"Tap with 3 fingers to continue" At:ccpAdd(m.position,ccp(0,80)) For:3 FontSize:25/gview.scale];
	}
	
	NSMutableArray *allmp = [NSMutableArray arrayWithArray:dl.allMapPoints];
	int mpcounter = 0;
	int drawn = 0;
	for(NSMutableDictionary *apoint in allmp){
		float per = [[apoint objectForKey:kMPercent] floatValue];
		if(per>=1 && m.visible){
			mpcounter++;
			drawn++;
			continue;
		}
			
		CGPoint p = [[apoint objectForKey:kMPosition] CGPointValue];
		float r = [[apoint objectForKey:kMRadius] floatValue];
		
		if(m.visible && ccpDistance(m.position, p)<=r && m.canBeHealed){
			per = per + 0.005;
		}
		
		if(m.visible==NO)
			per = per - 0.004;
		
		if(per>1)
			per = 1;
		if(per<0)
			per = 0;
		
		NSMutableDictionary *anewp = [NSMutableDictionary dictionaryWithDictionary:apoint];
		[anewp setObject:[NSNumber numberWithFloat:per] forKey:kMPercent];
		[dl.allMapPoints replaceObjectAtIndex:mpcounter withObject:anewp];
		mpcounter++;
	}
	[theUIView.numberOfHex setString:[NSString stringWithFormat:@"%d/6",drawn]];
	[theUIView.leftEnemies setString:[NSString stringWithFormat:@"D:%d",d]];
	if(t%60<10)
		[theUIView.locationBoard setString:[NSString stringWithFormat:@"%d:0%d",t/60,t%60]];
	else 
		[theUIView.locationBoard setString:[NSString stringWithFormat:@"%d:%d",t/60,t%60]];

	if(drawn==6){
		[[DataOfTheAncients sharedDOTA] addK:0 De:0 T:t];
		[theUIView showVictory];
		return;
	}
	
	for(ApostleBasis *e in es){
		[e doAIActionTo:m AtSpeed:10];	
	}
	
	whetherAlive = m.visible;
	if(i%20==0)
		t++;
}

-(void)shootModeupdate{
	if(i>=300)i=0;
	i++;
	
	GameView *gview = [GameView sharedGameView];
	DataLayer *dl = [DataLayer sharedDataLayer];
	ApostleBasis *m = gview.mc;
	NSMutableArray *es = gview.allEnemies;
	
	if(m.visible){
		m.position = gview.centerPosition;
		
		if(actionController.isMoving){
			[m turnto:actionController.angle atSpeed:5];
			if(!m.haveShield){
				[gview moveCenterAt:actionController.speed WithRotation:actionController.angle];	
				[[AnimationLayer sharedAnimationLayer] showMoveEffectAt:m.position WithRotation:actionController.angle Speed:actionController.speed];
				
			}
		}
		
		else [[AnimationLayer sharedAnimationLayer] stopMoveEffect];
		
		if(actionController.isScaling){
			if(firecounter%30==0){
				CGRect ranc = m.selfcolor;
				ranc.origin.x*=CCRANDOM_0_1()*0.7+0.3;
				ranc.origin.y*=CCRANDOM_0_1()*0.7+0.3;
				ranc.size.width*=CCRANDOM_0_1()*0.7+0.3;
				[m shoot:ranc From:m.position Angle:m.rotation Speed:60 Range:m.attackDistance Damage:firecounter*20 Shape:2 Radius:m.selfsize*0.8];
			}
			firecounter++;
		}
		else 
			firecounter = 0;
	}
	else {
		[[AnimationLayer sharedAnimationLayer] stopMoveEffect];
		[gview moveCenterAt:actionController.speed WithRotation:actionController.angle];
		if(actionController.shoot){
			m.currentEnergy = m.maxEnergy;
			m.position = gview.centerPosition;
			[m becomeActiveExplore];
		}
	}
	
	if(whetherAlive==YES && m.visible==NO){
		d++;
		[[AnimationLayer sharedAnimationLayer] addMessage:@"Tap with 3 fingers to continue" At:ccpAdd(m.position,ccp(0,80)) For:3 FontSize:25/gview.scale];
	}
	
	theUIView.currentE = m.currentEnergy;
	theUIView.maxE = m.maxEnergy;
	[theUIView.locationBoard setString:[NSString stringWithFormat:@"K/D:%d/%d",k,d]];
	[theUIView.leftEnemies setString:[NSString stringWithFormat:@"Wave:%d",w]];
	
	k+=[gview removeTheDead];
	dl.kills = k;
	dl.deathes = d;

	if(i==1){
		if([es count]<1){
			for(int wi=0;wi<w/6+1;wi++){
				float rxadd = (CCRANDOM_0_1()-0.5)*screenSize.width*2;
				float ryadd = (CCRANDOM_0_1()-0.5)*screenSize.height*2;
				CGPoint ranp = ccpAdd(m.position, ccp(rxadd,ryadd));
				[gview addEnemyAt:ranp Type:w%6+wi+1 Number:1];
			}
			w++;
		}
	}
	for(ApostleBasis *e in es){
		[e doAIActionTo:m AtSpeed:14];	
	}
	whetherAlive = m.visible;	
}

-(void)vsModeupdate{
	GameView *gview = [GameView sharedGameView];
	DataLayer *dl = [DataLayer sharedDataLayer];
	ApostleBasis *m = gview.mc;
	
	if(dl.gameState==kStatePicker || dl.gameState==kStateStartGame){
		i=-1;
		[gview removeAllEnemies];
	}
	
	if(i==0 && dl.gameState==kStateMultiplayer){
		NSDictionary *dic = [m selfFeatures];
		NSData *data = [NSKeyedArchiver archivedDataWithRootObject:dic];
		[dl sendNetworkPacket:dl.gameSession packetID:0 withData:data ofLength:0 reliable:NO];
	}
	else if(dl.gameState==kStateMultiplayer){
		NSDictionary *dic = [m currentStatus];
		NSData *data = [NSKeyedArchiver archivedDataWithRootObject:dic];
		[dl sendNetworkPacket:dl.gameSession packetID:0 withData:data ofLength:0 reliable:NO];
	}
		
	if(i>=300)i=0;
	i++;
	
	if(m.visible){
		m.position = gview.centerPosition;
		
		if(actionController.isMoving){
			[m turnto:actionController.angle atSpeed:5];
			if(!m.haveShield){
				[gview moveCenterAt:actionController.speed WithRotation:actionController.angle];	
				[[AnimationLayer sharedAnimationLayer] showMoveEffectAt:m.position WithRotation:actionController.angle Speed:actionController.speed];
				
			}
		}
		
		else [[AnimationLayer sharedAnimationLayer] stopMoveEffect];
	}
	else {
		[[AnimationLayer sharedAnimationLayer] stopMoveEffect];
		[gview moveCenterAt:actionController.speed WithRotation:actionController.angle];
		if(actionController.shoot){
			m.currentEnergy = m.maxEnergy;
			m.position = gview.centerPosition;
			[m becomeActiveExplore];
		}
	}
	
	if(whetherAlive==YES && m.visible==NO){
		d++;
		[[AnimationLayer sharedAnimationLayer] addMessage:@"Tap with 3 fingers to continue" At:ccpAdd(m.position,ccp(0,80)) For:3 FontSize:25/gview.scale];
		
		NSDictionary *dic = [NSDictionary dictionaryWithObjectsAndKeys:
							 [NSNumber numberWithInteger:NETWORK_COINTOSS],kDataType,
							 [NSNumber numberWithInt:d],@"dtimes",
							 nil];
		NSData *data = [NSKeyedArchiver archivedDataWithRootObject:dic];
		
		[dl sendNetworkPacket:dl.gameSession packetID:0 withData:data ofLength:0 reliable:NO];
	}
	
	theUIView.currentE = m.currentEnergy;
	theUIView.maxE = m.maxEnergy;
	[theUIView.locationBoard setString:[NSString stringWithFormat:@"K/D:%d/%d",dl.kills,d]];
	NSString *state;
	if(dl.gameState==kStateStartGame || dl.gameState==kStatePicker){
		state = [NSString stringWithFormat:@"Disconnect"];
	}
	if(dl.gameState==kStateMultiplayer){
		state = [NSString stringWithFormat:@"Connected"];
	}
	[theUIView.leftEnemies setString:state];
	
	whetherAlive = m.visible;		
}

-(void)achieveModeupdate{
	GameView *gview = [GameView sharedGameView];
	
	if(actionController.isMoving)
		[gview moveCenterAt:actionController.speed*2 WithRotation:actionController.angle];
	if(actionController.isScaling)
		[gview zoomGameViewWith:actionController.zoomfactor];
	if(actionController.shoot){
		[[CCDirector sharedDirector] popScene];
		return;
	}
}

@end
