//
//  GameUI.m
//  ColorsWar
//
/*
 * HAZ™ Dev Group: http://imhaz.com
 *
 * Copyright (c) 2007-2011 HAZ™ Inc.
 *
 * Written by Hercy Chang 2011-12-08
 *
 */
#import "MainMenu.h"
#import "GameUI.h"
#import "DataLayer.h"
#import "GameView.h"
#import "CWColor.h"

@interface GameUI (Private)

-(void)addColorLayerForUIObjects;
-(void)addLabelBoard;
-(void)addSPBoard;
-(void)addPauseButton;
-(void)addSkillBar;
@end

@implementation GameUI

@synthesize colorToDrawUI;
@synthesize currentE,maxE;
@synthesize numberOfHex;
@synthesize locationBoard;
@synthesize leftEnemies;

-(void)addColorLayerForUIObjects{
	CCLayerColor *colorlayer1 = [CCLayerColor layerWithColor:ccc4(0,0,0,150) width:screenSize.width height:screenSize.height/10];
	colorlayer1.position = ccp(0,screenSize.height*9/10);
	CCLayerColor *colorlayer2 = [CCLayerColor layerWithColor:ccc4(0,0,0,150) width:screenSize.width height:screenSize.height/8];
	
	[self addChild:colorlayer1 z:-1];
	[self addChild:colorlayer2 z:-1];
}

-(void)addLabelBoard{
	numberOfHex = [CCLabelTTF labelWithString:@"" fontName:@"Verdana" fontSize:screenSize.height/18];
	numberOfHex.position = ccp(screenSize.width*5/48,screenSize.height*30/32);
	[self addChild:numberOfHex];
	
	locationBoard = [CCLabelTTF labelWithString:@"" fontName:@"Verdana" fontSize:screenSize.height/18];
	locationBoard.position = ccp(screenSize.width*21/24,screenSize.height*30/32);
	[self addChild:locationBoard];
	
	leftEnemies = [CCLabelTTF labelWithString:@"" fontName:@"Verdana" fontSize:screenSize.height/24];
	leftEnemies.position = ccp(screenSize.width*20/24,screenSize.height*2/32);
	[self addChild:leftEnemies];
}

-(void)addSPBoard{
	CCMenuItem *sp = [CCMenuItemFont itemFromString:@"   " target:self selector:@selector(showSPInformation:)];
	[sp setScale:4];
	CCMenu *spbutton = [CCMenu menuWithItems:sp,nil];
	spbutton.position = ccp(screenSize.width/30,screenSize.height*30/32);
	[self addChild:spbutton];
}
-(IBAction)showSPInformation:(id)sender{
	if(drawSPBoard){
		drawSPBoard = NO;
		[self removeChildByTag:1 cleanup:YES];
	}
	else {
		drawSPBoard = YES;
		CCLayerColor *backlayertag1;
		backlayertag1 = [CCLayerColor layerWithColor:ccc4(0,0,0,150) width:screenSize.width height:screenSize.height*31/40];
		backlayertag1.position = ccp(0,screenSize.height/8);
		[self addChild:backlayertag1 z:-1 tag:1];				
		
	}
}

-(void)addPauseButton{
	CCMenuItem *pause = [CCMenuItemFont itemFromString:@" || " target:self selector:@selector(gamepause:)];
	CCMenu *p = [CCMenu menuWithItems:pause,nil];
	p.position = ccp(screenSize.width*23/24,screenSize.height/18);
	[self addChild:p];
}

-(IBAction)gamepause:(id)sender{
	if([CCDirector sharedDirector].isPaused)
		return;
	[[CCDirector sharedDirector] pause];
	CCNode *menu = [self getChildByTag:0];
	if(!menu){
		CCMenuItem *re = [CCMenuItemFont itemFromString:@"Resume" target:self selector:@selector(resume:)];
		CCMenuItem *back = [CCMenuItemFont itemFromString:@"Main Menu" target:self selector:@selector(backtomain:)];
		CCMenu *p = [CCMenu menuWithItems:re,back,nil];
		[p alignItemsVerticallyWithPadding:screenSize.height/5];
		p.position = ccp(screenSize.width/2,screenSize.height/2);
		[self addChild:p z:99 tag:0];		
	}
}

-(IBAction)backtomain:(id)sender{
	[[CCDirector sharedDirector] resume];
	
	DataLayer *dl = [DataLayer sharedDataLayer];
	[dl addKDToDOTA];
	dl.gameState = kStateStartGame;
	
	[[CCDirector sharedDirector] popScene];
}

-(IBAction)resume:(id)sender{
	[self removeChildByTag:0 cleanup:YES];
	[[CCDirector sharedDirector] resume];
	
	DataLayer *dl = [DataLayer sharedDataLayer];
	if (dl.gameMode==GameModeChallenge && dl.gameState==kStateStartGame) {
		id action1 = [CCDelayTime actionWithDuration:0.1];
		id action2 = [CCCallFunc actionWithTarget:dl selector:@selector(startPicker)];
		id actionseq = [CCSequence actionOne:action1 two:action2];
		[self runAction:actionseq];
	}
}

-(void)addSkillBar{
	SkillsButton *sbo = [SkillsButton node];
	[self addChild:sbo];
}

-(void)dealloc{
	[super dealloc];
}

-(id)init{
	if((self=[super init])){
		screenSize = [[CCDirector sharedDirector] winSize];
		currentE = maxE = 1000;
		DataLayer *dl = [DataLayer sharedDataLayer];
		w = dl.mapSize.width*dl.length;
		h = dl.mapSize.height*dl.length;

		colorToDrawUI = CGRectMake(1, 1, 1, 1);
		drawSPBoard = NO;
		[self addColorLayerForUIObjects];
		[self addLabelBoard];
		[self addSPBoard];
		[self addPauseButton];
		[self addSkillBar];
	}
	return self;
}
void drawEnergyBar( CGPoint s, CGPoint e, float width, float percent, CGRect startColor , CGRect endColor)
{
	float length = (e.x-s.x)*percent;
	ccVertex2F vertices[4] = {
		{s.x,s.y+width/2},
		{s.x,s.y-width/2},
		{s.x+length,e.y+width/2},
		{s.x+length,e.y-width/2},
	};
	CGRect mixed;
	if(percent==0)
		mixed = endColor;
	else 
		mixed = CGRectMix(startColor, endColor, 1/percent);
	
	ccVertex2F colors[8] = {
		{startColor.origin.x, startColor.origin.y},
		{startColor.size.width, startColor.size.height},
		{startColor.origin.x, startColor.origin.y},
		{startColor.size.width, startColor.size.height},
		{mixed.origin.x, mixed.origin.y},
		{mixed.size.width, mixed.size.height},
		{mixed.origin.x, mixed.origin.y},
		{mixed.size.width, mixed.size.height},
	};
	
	// Default GL states: GL_TEXTURE_2D, GL_VERTEX_ARRAY, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	// Needed states: GL_VERTEX_ARRAY, 
	// Unneeded states: GL_TEXTURE_2D, GL_TEXTURE_COORD_ARRAY, GL_COLOR_ARRAY	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	
	// restore default state
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);		
}
void drawNumOfHex( CGPoint center, float r,CGRect color1,CGRect color2 )
{
	const float coef = (float)M_PI/3;
	
	GLfloat *vertices = calloc( sizeof(GLfloat)*2*8, 1);
	GLfloat *colors = calloc( sizeof(GLfloat)*4*8, 1);
	if( ! vertices || ! colors)
		return;
	
	vertices[0] = center.x;
	vertices[1] = center.y;
	colors[0] = color1.origin.x;
	colors[1] = color1.origin.y;
	colors[2] = color1.size.width;
	colors[3] = color1.size.height;
	
	for(NSUInteger i=1;i<8;i++)
	{
		float rads = (i-1)*coef;
		GLfloat ix = r * sinf(rads) + center.x;
		GLfloat iy = r * cosf(rads) + center.y;
		
		vertices[i*2] = ix * CC_CONTENT_SCALE_FACTOR();
		vertices[i*2+1] = iy * CC_CONTENT_SCALE_FACTOR();
		
		colors[i*4] = color2.origin.x;
		colors[i*4+1] = color2.origin.y;
		colors[i*4+2] = color2.size.width;
		colors[i*4+3] = color2.size.height;
	}
	
	// Default GL states: GL_TEXTURE_2D, GL_VERTEX_ARRAY, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	// Needed states: GL_VERTEX_ARRAY, 
	// Unneeded states: GL_TEXTURE_2D, GL_TEXTURE_COORD_ARRAY, GL_COLOR_ARRAY	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	//glDisableClientState(GL_COLOR_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_FAN, 0, 8);
	
	// restore default state
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);	
	
	free( vertices );
	free( colors );
}
void drawConnect(CGPoint sp,CGPoint ep,CGRect startc,CGRect endc,CGRect edgecolor,float width)
{
	float anglestoe = AngleOfTwoPoints(sp, ep);
	float radfactor = (float)M_PI/180;
	
	GLfloat vertices[12];
	GLfloat colors[24];
	
	float dx1 = width/2*sinf((anglestoe+90)*radfactor);
	float dy1 = width/2*cosf((anglestoe+90)*radfactor);
	float dx2 = width/2*sinf((anglestoe+270)*radfactor);
	float dy2 = width/2*cosf((anglestoe+270)*radfactor);
	
	vertices[0] = sp.x+dx1;
	vertices[1] = sp.y+dy1;
	vertices[2] = ep.x+dx1;
	vertices[3] = ep.y+dy1;
	vertices[4] = sp.x;
	vertices[5] = sp.y;
	vertices[6] = ep.x;
	vertices[7] = ep.y;
	vertices[8] = sp.x+dx2;
	vertices[9] = sp.y+dy2;
	vertices[10] = ep.x+dx2;
	vertices[11] = ep.y+dy2;
	
	colors[0] = colors[4] = colors[16] = colors[20] = edgecolor.origin.x;
	colors[1] = colors[5] = colors[17] = colors[21] = edgecolor.origin.y;
	colors[2] = colors[6] = colors[18] = colors[22] = edgecolor.size.width;
	colors[3] = colors[7] = colors[19] = colors[23] = edgecolor.size.height;
	
	colors[8] = startc.origin.x;
	colors[9] = startc.origin.y;
	colors[10] = startc.size.width;
	colors[11] = startc.size.height;
	
	colors[12] = endc.origin.x;
	colors[13] = endc.origin.y;
	colors[14] = endc.size.width;
	colors[15] = endc.size.height;
	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 6);
	
	// restore default state
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);		
	
}

-(void)draw{
	CGPoint hexpos = ccp(screenSize.width/30,screenSize.height*30/32);
	CGPoint start = ccp(screenSize.width*2/12,screenSize.height*30/32);
	CGPoint end = ccp(screenSize.width*8/12,screenSize.height*30/32);
	drawEnergyBar(start, end, 6, (float)currentE/(float)maxE, colorToDrawUI, CGRectMake(1, 1, 1, 1));
	
	if(drawSPBoard){
		DataLayer *dl = [DataLayer sharedDataLayer];
		GameView *gview = [GameView sharedGameView];
		
		drawNumOfHex(hexpos, screenSize.height/16, colorToDrawUI, BlackWithAlpha(0));
		
		int mpcounter = 0;
		for(NSMutableDictionary *apoint in dl.allMapPoints){
			float per = [[apoint objectForKey:kMPercent] floatValue];				
			CGPoint p = [[apoint objectForKey:kMPosition] CGPointValue];
			CGPoint cup = ccp(p.x/w*screenSize.width,p.y/h*screenSize.height*31/40+screenSize.height/8);
			CGRect c = [[apoint objectForKey:kMColor] CGRectValue];
			CGRect cuc = CGRectMultiple(c, per);

			if(mpcounter%2==0){
				p1 = cup;
				c1 = cuc;
			}else {
				p2 = cup;
				c2 = cuc;
			}
			if(mpcounter){
				drawConnect(p1, p2, c1, c2, BlackWithAlpha(0), 8);
			}
			if(per>0){
				drawNumOfHex(cup, 8,cuc , BlackWithAlpha(0));
			}
			if(mpcounter==0 && per<0.8){
				drawNumOfHex(cup, 8,WhiteWithAlpha(0) , BlackWithAlpha(0));
			}
			mpcounter++;
		}
		CGPoint mpos = gview.centerPosition;
		CGPoint converted1;
		converted1 = ccp(mpos.x/w*screenSize.width,mpos.y/h*screenSize.height*31/40+screenSize.height/8);
		
		drawNumOfHex(converted1, 5,colorToDrawUI , BlackWithAlpha(0));
		if(dl.gameMode==GameModeChallenge){
			NSMutableArray *es =  gview.allEnemies;
			if([es count]==1){
				ApostleBasis *e = [es objectAtIndex:0];
				CGPoint epos = e.position;
				CGPoint converted2 = ccp(epos.x/w*screenSize.width,epos.y/h*screenSize.height*31/40+screenSize.height/8);
				drawNumOfHex(converted2, 5,colorToDrawUI , BlackWithAlpha(0));
			}
		}
	}
	else{
		drawNumOfHex(hexpos, screenSize.height/24, colorToDrawUI, BlackWithAlpha(0));
	}
} 
 
-(void)showVictory{
	[[CCDirector sharedDirector] pause];
	CCNode *menu = [self getChildByTag:99];
	if(!menu){
		CCMenuItem *con1 = [CCMenuItemFont itemFromString:@"Mission Completed" target:self selector:@selector(backtomain:)];
		CCMenu *p = [CCMenu menuWithItems:con1,nil];
		p.position = ccp(screenSize.width/2,screenSize.height/2);
		[self addChild:p z:99 tag:99];	
		[[DataLayer sharedDataLayer] addMapPointsToDOTA];
	}
}

 

@end
