/*
 * HAZ™ Dev Group: http://imhaz.com
 *
 * Copyright (c) 2007-2011 HAZ™ Inc.
 *
 * Written by Hercy Chang 2011-12-08
 *
 */

#define kEPosition @"enemyPosition"
#define kEType @"enemyType"

#import "GameView.h"
#import "GameMap.h"
#import "DataLayer.h"
#import "DataOfTheAncients.h"
#import "CWColor.h"
#import "AnimationLayer.h"

@interface GameView (Private)

-(void)ExploreMode;
-(void)ShootMode;
-(void)vsMode;
-(void)AchieveMode;
-(void)addGameMap;
-(void)addAnimationLayer;

@end


@implementation GameView

@synthesize centerPosition;
@synthesize mapSize;
@synthesize mc;
@synthesize allEnemies;

static GameView* instanceOfGameView;
+(GameView*) sharedGameView
{
	NSAssert(instanceOfGameView != nil, @"GameScene instance not yet initialized!");
	return instanceOfGameView;
}

-(void)ExploreMode{
	[self addGameMap];
	[self addAnimationLayer];
	[allEnemies removeAllObjects];
	
	DataLayer *dl = [DataLayer sharedDataLayer];
	
	mapSize = CGSizeMake(dl.mapSize.width*dl.length, dl.mapSize.height*dl.length);	
	[dl removeAllMapPoints];
	for(int m=0;m<6;m++){
		int rx = CCRANDOM_0_1()*dl.mapSize.width;
		int ry = CCRANDOM_0_1()*dl.mapSize.height;
		CGPoint rp = ccp(rx*dl.length,ry*dl.length);
		CGRect rc = CGRectMake(0.3+CCRANDOM_0_1()*0.7,0.3+CCRANDOM_0_1()*0.7,0.3+CCRANDOM_0_1()*0.7,1);
		[dl addPointAt:rp Color:rc BColor:BlackWithAlpha(0) Radius:dl.length Type:6];
		[self addEnemyAt:rp Type:m+1 Number:1];
	}
	DataOfTheAncients *dota = [DataOfTheAncients sharedDOTA];
	ApostleBasis *cm = dota.theship;
	mc = [ApostleBasis node];
	[mc pasteFrom:cm];
	[mc calculateSize];
	
	[self addChild:mc];
	mc.visible = NO;
	[mc becomeActiveExplore];
	
	CGPoint mapcenter = ccp(dl.mapSize.width*dl.length/2,dl.mapSize.height*dl.length/2);
	mc.position = mapcenter;
	centerPosition = mapcenter;
	[self setCameraCenterAt:mapcenter];
	[self zoomGameViewWith:(screenSize.height+screenSize.width)/mc.visualDistance/6];
	
	[[AnimationLayer sharedAnimationLayer] addMessage:@"Game Start" At:mapcenter For:3 FontSize:80/self.scale];
}

-(void)ShootMode{
	[self addAnimationLayer];
	[self addGameMap];
	[allEnemies removeAllObjects];
	DataLayer *dl = [DataLayer sharedDataLayer];
	[dl removeAllMapPoints];
	DataOfTheAncients *dota = [DataOfTheAncients sharedDOTA];
	ApostleBasis *cm = dota.theship;
	mc = [ApostleBasis node];
	[mc pasteFrom:cm];
	[mc calculateSize];

	[self addChild:mc];
	mc.visible = NO;
	[mc becomeActiveExplore];
	
	CGPoint mapcenter = ccp(dl.mapSize.width*dl.length/2,dl.mapSize.height*dl.length/2);
	mc.position = mapcenter;
	centerPosition = mapcenter;
	[self setCameraCenterAt:mapcenter];
	
	mapSize = CGSizeMake(dl.mapSize.width*dl.length, dl.mapSize.height*dl.length);	
	[self zoomGameViewWith:(screenSize.height+screenSize.width)/mc.visualDistance/6];
}

-(void)vsMode{
	[self addAnimationLayer];
	[self addGameMap];
	[allEnemies removeAllObjects];
	DataLayer *dl = [DataLayer sharedDataLayer];
	[dl removeAllMapPoints];
	DataOfTheAncients *dota = [DataOfTheAncients sharedDOTA];
	ApostleBasis *cm = dota.theship;
	mc = [ApostleBasis node];
	[mc pasteFrom:cm];
	[mc calculateSize];
	
	[self addChild:mc];
	mc.visible = NO;
	[mc becomeActiveExplore];
	
	CGPoint mapcenter = ccp(dl.mapSize.width*dl.length/2,dl.mapSize.height*dl.length/2);
	mc.position = mapcenter;
	centerPosition = mapcenter;
	[self setCameraCenterAt:mapcenter];
	
	mapSize = CGSizeMake(dl.mapSize.width*dl.length, dl.mapSize.height*dl.length);	
	[self zoomGameViewWith:(screenSize.height+screenSize.width)/mc.visualDistance/6];	
}

-(void)AchieveMode{
	[self addGameMap];
	[self addAnimationLayer];
	DataOfTheAncients *dota = [DataOfTheAncients sharedDOTA];
	DataLayer *dl = [DataLayer sharedDataLayer];
	int adp = [dota.drawnPoints count]-3;
	if(adp<0)
		adp=0;
	
	float xadd;
	if((adp/10)%2==0)
		xadd = adp%10*screenSize.width;
	else 
		xadd = (9-adp%10)*screenSize.width;
	
	float yadd = adp/10*screenSize.height;
	
	CGPoint mapcenter = ccp(xadd+dl.length/2,yadd+dl.length/2);
	centerPosition = mapcenter;
	[self setCameraCenterAt:mapcenter];
	
	mapSize = CGSizeMake(screenSize.width*10, screenSize.height*10);	
	
	[[AnimationLayer sharedAnimationLayer] addMessage:@"Use three fingers to go back" At:mapcenter For:10 FontSize:screenSize.height/8];
}

-(void)addGameMap{
	GameMap *map = [GameMap node];
	[self addChild:map z:-1];
}

-(void)addAnimationLayer{
	AnimationLayer *aml = [AnimationLayer node];
	[self addChild:aml z:0];
}

-(void)dealloc{
	[allEnemies release];
	[super dealloc];
}	

-(id)init{
	if((self=[super init])){
		DataLayer *dl = [DataLayer sharedDataLayer];
		instanceOfGameView = self;
		allEnemies = [[NSMutableArray alloc] init];
		screenSize = [[CCDirector sharedDirector] winSize];
		switch (dl.gameMode) {
			case GameModeExplore:
				[self ExploreMode];
				break;
			case GameModeShoot:
				[self ShootMode];
				break;
			case GameModeFind:
				[self AchieveMode];
				break;
			case GameModeChallenge:
				[self vsMode];
				break;

			default:
				break;
		}
		
		}
	return self;
}

-(void)removeAllEnemies{
	for(ApostleBasis *e in allEnemies){
		[e removeFromParentAndCleanup:YES];
	}
	[allEnemies removeAllObjects];
}

-(int)removeTheDead{
	int ecounter=0;
	int dnum = 0;
	NSMutableArray *ae = [NSMutableArray arrayWithArray:allEnemies];
	for(ApostleBasis *e in ae){
		if(e.visible==NO){
			dnum+=1;
			[[AnimationLayer sharedAnimationLayer] addMessage:@"BOOM!" At:e.position For:1 FontSize:20/self.scale];
			[[AnimationLayer sharedAnimationLayer] showExplosionAt:e.position WithTime:0.5 Color:WhiteWithAlpha(1) WaveSpeed:50];
			[e removeFromParentAndCleanup:YES];
			[allEnemies removeObjectAtIndex:ecounter];
		}
		else ecounter++;
	}
	return dnum;
}


-(void)addEnemyAt:(CGPoint)pos Type:(int)etype Number:(int)num{
	for(int i=0;i<num;i++){
		ApostleBasis *newenemy = [ApostleBasis node];
		newenemy.startPosition = newenemy.position = pos;
		newenemy.AIType = (etype-1)%6+1;
		CGRect randomc = ColorWithCode((etype-1)%6, 1);
		randomc.origin.x*=(CCRANDOM_0_1()*0.7+0.3);
		randomc.origin.y*=(CCRANDOM_0_1()*0.7+0.3);
		randomc.size.width*=(CCRANDOM_0_1()*0.7+0.3);
		
		newenemy.selfcolor = randomc;
		
		newenemy.rotation = CCRANDOM_0_1()*360;
		[newenemy changeShape];
		switch (newenemy.AIType) {
			case 1:
				newenemy.soulSpeed = 400;
				break;
			case 2:
				newenemy.drawSpeed = 20;
				break;
			case 3:
				newenemy.currentEnergy = newenemy.maxEnergy = 10000;
				break;
			case 4:
				newenemy.attackDistance = 600;
				break;
			case 5:
				newenemy.drawSpeed = 25;
				break;
			case 6:
				newenemy.drawSpeed = 30;
				newenemy.attackDistance = 800;
				break;
			default:
				break;
		}
		[allEnemies addObject:newenemy];
		[self addChild:newenemy];
		[newenemy becomeActiveExplore];
		[[AnimationLayer sharedAnimationLayer] showExplosionAt:pos WithTime:3.0 Color:ColorWithCode(etype-1, 0) WaveSpeed:-1];
	}
}

-(CGPoint)posOfEnemyAtIndex:(int)i{
	CGPoint p = ccp(-100,-10);
	int total = [allEnemies count];
	
	if(i<=total-1 && i>=0){
		ApostleBasis *e = [allEnemies objectAtIndex:i];
		if(e.visible)
			p = e.position;
	}
	
	return p;
}

-(void)moveCenterAt:(float)speed WithRotation:(float)angle{
	float mapWidth = self.mapSize.width;
	float mapHeight = self.mapSize.height;
	float xmove,ymove;
	xmove = speed*sin(angle/360*2*3.1415926);
	ymove = speed*cos(angle/360*2*3.1415926);
	
	CGPoint start = centerPosition;
	CGPoint end;
	end = ccp(start.x+xmove,start.y+ymove);
	
	if((end.x<0||end.x>mapWidth)&&(end.y<0||end.y>mapHeight)){
		return;
	}
	if(end.x<0||end.x>mapWidth){
		centerPosition = ccp(end.x-xmove,end.y);
	}
	else if(end.y<0||end.y>mapHeight){
		centerPosition = ccp(end.x,end.y-ymove);
	}
	else {
		centerPosition = end;
	};
	[self setCameraCenterAt:centerPosition];
}

-(void)zoomGameViewWith:(float)zoomfactor{
	self.scale = zoomfactor;
	[self setCameraCenterAt:centerPosition];
}

-(void)setCameraCenterAt:(CGPoint)point{
	float zoomf = self.scale;
	CGPoint addition = ccp(screenSize.width/2*(1-self.scale),screenSize.height/2*(1-self.scale));
	CGPoint buttomleftpos = ccp(screenSize.width/2-point.x*zoomf,screenSize.height/2-point.y*zoomf);
	self.position = ccpSub(buttomleftpos, addition);	
}



@end
