//
//  SkillsButton.m
//  ColorsWar
//
/*
 * HAZ™ Dev Group: http://imhaz.com
 *
 * Copyright (c) 2007-2011 HAZ™ Inc.
 *
 * Written by Hercy Chang 2011-12-08
 *
 */
/*
 Draw six skill buttons on 
 
 (width*1/16,height*1/16)
 (width*3/16,height*1/16)
 (width*5/16,height*1/16)
 (width*7/16,height*1/16)
 (width*9/16,height*1/16)
 (width*11/16,height*1/16)
 
 radius = height/16;
 
 Button's status (CGRect):
{0,0,0,0}(black) not pressed
selfcolor        pressed
 */

#import "SkillsButton.h"
#import "GameView.h"
#import "CWColor.h"

@interface SkillsButton (Private)

-(void)statusSwitchAt:(int)i;

@end


@implementation SkillsButton

void drawSkillButton( CGPoint center, float r,CGRect color1,CGRect color2 )
{
	const float coef = (float)M_PI/3;
	
	GLfloat *vertices = calloc( sizeof(GLfloat)*2*8, 1);
	GLfloat *colors = calloc( sizeof(GLfloat)*4*8, 1);
	if( ! vertices || ! colors)
		return;
	
	vertices[0] = center.x;
	vertices[1] = center.y;
	colors[0] = color1.origin.x;
	colors[1] = color1.origin.y;
	colors[2] = color1.size.width;
	colors[3] = color1.size.height;
	
	for(NSUInteger i=1;i<8;i++)
	{
		float rads = (i-1)*coef;
		GLfloat ix = r * sinf(rads) + center.x;
		GLfloat iy = r * cosf(rads) + center.y;
		
		vertices[i*2] = ix * CC_CONTENT_SCALE_FACTOR();
		vertices[i*2+1] = iy * CC_CONTENT_SCALE_FACTOR();
		
		colors[i*4] = color2.origin.x;
		colors[i*4+1] = color2.origin.y;
		colors[i*4+2] = color2.size.width;
		colors[i*4+3] = color2.size.height;
	}
	
	// Default GL states: GL_TEXTURE_2D, GL_VERTEX_ARRAY, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	// Needed states: GL_VERTEX_ARRAY, 
	// Unneeded states: GL_TEXTURE_2D, GL_TEXTURE_COORD_ARRAY, GL_COLOR_ARRAY	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	//glDisableClientState(GL_COLOR_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_FAN, 0, 8);
	
	// restore default state
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);	
	
	free( vertices );
	free( colors );
}
void SkillButtonChosen( CGPoint center,float r1, float r2, CGRect color1, CGRect color2, int numofpoint)
{
	const float coef = (float)M_PI*2/numofpoint;
	
	GLfloat *vertices = calloc( sizeof(GLfloat)*2*(numofpoint+1)*2, 1);
	GLfloat *colors = calloc( sizeof(GLfloat)*4*(numofpoint+1)*2, 1);
	if( ! vertices || ! colors)
		return;
	
	vertices[0] = center.x;
	vertices[1] = center.y+r1;
	vertices[2] = center.x;
	vertices[3] = center.y+r2;
	colors[0] = color1.origin.x;
	colors[1] = color1.origin.y;
	colors[2] = color1.size.width;
	colors[3] = color1.size.height;
	colors[4] = color2.origin.x;
	colors[5] = color2.origin.y;
	colors[6] = color2.size.width;
	colors[7] = color2.size.height;
	
	for(NSUInteger i=1;i<(numofpoint+1);i++)
	{
		float rads = i*coef;
		GLfloat ix1 = r1 * sinf(rads) + center.x;
		GLfloat iy1 = r1 * cosf(rads) + center.y;
		GLfloat ix2 = r2 * sinf(rads) + center.x;
		GLfloat iy2 = r2 * cosf(rads) + center.y;
		
		vertices[i*4] = ix1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+1] = iy1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+2] = ix2 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+3] = iy2 * CC_CONTENT_SCALE_FACTOR();
		
		colors[i*8] = color1.origin.x;
		colors[i*8+1] = color1.origin.y;
		colors[i*8+2] = color1.size.width;
		colors[i*8+3] = color1.size.height;
		
		colors[i*8+4] = color2.origin.x;
		colors[i*8+5] = color2.origin.y;
		colors[i*8+6] = color2.size.width;
		colors[i*8+7] = color2.size.height;
	}
	
	// Default GL states: GL_TEXTURE_2D, GL_VERTEX_ARRAY, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	// Needed states: GL_VERTEX_ARRAY, 
	// Unneeded states: GL_TEXTURE_2D, GL_TEXTURE_COORD_ARRAY, GL_COLOR_ARRAY	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	//glDisableClientState(GL_COLOR_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, (numofpoint+1)*2);
	
	// restore default state
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);	
	
	free( vertices );
	free( colors );
	
}

-(void)draw{
	float width16 = screenSize.width/16;
	float height16 = screenSize.height/16;

	for(int i=0;i<6;i++){
		CGPoint spos = ccp(width16*(2*i+1),height16);
		CGRect scolor = ColorWithCode(i, 1);
		float realr = height16;
		drawSkillButton(spos, realr, scolor, BlackWithAlpha(0));
		if(status[i]==1){
			SkillButtonChosen(spos, realr*0.75, realr*0.9, BlackWithAlpha(0), scolor, 64);
			SkillButtonChosen(spos, realr*0.9, realr, scolor, BlackWithAlpha(0), 64);
		}
	}
}
-(void)dealloc{
	[super dealloc];
}

-(id)init{
	if((self=[super init])){
		screenSize = [[CCDirector sharedDirector] winSize];
		self.isTouchEnabled = YES;

		for(int i=0;i<6;i++)
			status[i] = 0;
		
	}
	return self;
}

-(void)statusSwitchAt:(int)i{
	if(i<0 || i>5)
		return;
	
	if(status[i]==0)
		status[i]=1;
	else 
		status[i]=0;
}

-(void)ccTouchesBegan:(NSSet*)touches withEvent:(UIEvent*)event{
	UITouch *touch = [touches anyObject];
	CGPoint location = [touch locationInView:[touch view]];
	CGPoint tp = [[CCDirector sharedDirector] convertToGL:location];
	int i=0;
	float width16 = screenSize.width/16;
	float height16 = screenSize.height/16;
	
	CGRect skbr = CGRectMake(0, 0, width16*12,height16*2);
	
	if(!CGRectContainsPoint(skbr, tp))
		return;
	else 
		i = (int)(tp.x/(width16*2));
	if(i<6){
		GameView *gview = [GameView sharedGameView];
		[self statusSwitchAt:i];
		int s = status[i];
		
		switch (i) {
			case 0:{
				if(s)
					gview.mc.canSee=YES;
				else 
					gview.mc.canSee=NO;
				break;}
			case 1:{
				if(s)
					gview.mc.isDrawing=YES;
				else 
					gview.mc.isDrawing=NO;
				break;}
			case 2:{
				if(s)
					gview.mc.canBeHealed=YES;
				else 
					gview.mc.canBeHealed=NO;
				break;}
			case 3:{
				if(s)
					gview.mc.soulMove=YES;
				else 
					gview.mc.soulMove=NO;
				break;}
			case 4:{
				if(s)
					gview.mc.isAttacking=YES;
				else 
					gview.mc.isAttacking=NO;
				break;}
			case 5:{
				if(s)
					gview.mc.haveShield=YES;
				else 
					gview.mc.haveShield=NO;
				break;}
			default:
				break;
		}		
	}
}
-(void)ccTouchesMoved:(NSSet*)touches withEvent:(UIEvent*)event{

}
-(void)ccTouchesEnded:(NSSet*)touches withEvent:(UIEvent*)event{
}
-(void)ccTouchesCancelled:(NSSet*)touches withEvent:(UIEvent*)event{
}

@end
