//
//  StartLayer.m
//  ColorsWar
//
/*
 * HAZ™ Dev Group: http://imhaz.com
 *
 * Copyright (c) 2007-2011 HAZ™ Inc.
 *
 * Written by Hercy Chang 2011-12-08
 *
 */

#import "StartLayer.h"
#import "MainMenu.h"
#import "CWColor.h"


@implementation StartLayer

+(id)scene{
	CCScene *sc = [CCScene node];
	StartLayer *sl = [StartLayer node];
	[sc addChild:sl];
	return sc;
}

void startHex( CGPoint center,float r1, float r2, CGRect color1, CGRect color2, int numofpoint)
{
	const float coef = (float)M_PI*2/numofpoint;
	
	GLfloat *vertices = calloc( sizeof(GLfloat)*2*(numofpoint+1)*2, 1);
	GLfloat *colors = calloc( sizeof(GLfloat)*4*(numofpoint+1)*2, 1);
	if( ! vertices || ! colors)
		return;
	
	vertices[0] = center.x;
	vertices[1] = center.y+r1;
	vertices[2] = center.x;
	vertices[3] = center.y+r2;
	colors[0] = color1.origin.x;
	colors[1] = color1.origin.y;
	colors[2] = color1.size.width;
	colors[3] = color1.size.height;
	colors[4] = color2.origin.x;
	colors[5] = color2.origin.y;
	colors[6] = color2.size.width;
	colors[7] = color2.size.height;
	
	for(NSUInteger i=1;i<(numofpoint+1);i++)
	{
		float rads = i*coef;
		GLfloat ix1 = r1 * sinf(rads) + center.x;
		GLfloat iy1 = r1 * cosf(rads) + center.y;
		GLfloat ix2 = r2 * sinf(rads) + center.x;
		GLfloat iy2 = r2 * cosf(rads) + center.y;
		
		vertices[i*4] = ix1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+1] = iy1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+2] = ix2 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+3] = iy2 * CC_CONTENT_SCALE_FACTOR();
		
		colors[i*8] = color1.origin.x;
		colors[i*8+1] = color1.origin.y;
		colors[i*8+2] = color1.size.width;
		colors[i*8+3] = color1.size.height;
		
		colors[i*8+4] = color2.origin.x;
		colors[i*8+5] = color2.origin.y;
		colors[i*8+6] = color2.size.width;
		colors[i*8+7] = color2.size.height;
	}
	
	// Default GL states: GL_TEXTURE_2D, GL_VERTEX_ARRAY, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	// Needed states: GL_VERTEX_ARRAY, 
	// Unneeded states: GL_TEXTURE_2D, GL_TEXTURE_COORD_ARRAY, GL_COLOR_ARRAY	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	//glDisableClientState(GL_COLOR_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, (numofpoint+1)*2);
	
	// restore default state
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);	
	
	free( vertices );
	free( colors );
	
}

void startCircle( CGPoint center,float r1, float r2, CGRect sc1,CGRect ec1 ,CGRect sc2,CGRect ec2, int numofpoint,float radstart,float radend)
{
	if(r1==r2)return;
	
	float coef = (radend-radstart)/numofpoint;
	float pif = (float)M_PI/180;
	
	GLfloat *vertices = calloc( sizeof(GLfloat)*2*(numofpoint+1)*2, 1);
	GLfloat *colors = calloc( sizeof(GLfloat)*4*(numofpoint+1)*2, 1);
	if( ! vertices || ! colors)
		return;
	
	for(NSUInteger i=0;i<(numofpoint+1);i++)
	{
		float rads = (i*coef + radstart)*pif;
		GLfloat ix1 = r1 * sinf(rads) + center.x;
		GLfloat iy1 = r1 * cosf(rads) + center.y;
		GLfloat ix2 = r2 * sinf(rads) + center.x;
		GLfloat iy2 = r2 * cosf(rads) + center.y;
		
		vertices[i*4] = ix1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+1] = iy1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+2] = ix2 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+3] = iy2 * CC_CONTENT_SCALE_FACTOR();
		
		float trf = sinf((float)i/numofpoint*3.1415926);
		CGRect color1 = CWColorTransform(sc1, ec1, trf);
		CGRect color2 = CWColorTransform(sc2, ec2, trf);
		
		colors[i*8] = color1.origin.x;
		colors[i*8+1] = color1.origin.y;
		colors[i*8+2] = color1.size.width;
		colors[i*8+3] = color1.size.height;
		
		colors[i*8+4] = color2.origin.x;
		colors[i*8+5] = color2.origin.y;
		colors[i*8+6] = color2.size.width;
		colors[i*8+7] = color2.size.height;
	}
	
	// Default GL states: GL_TEXTURE_2D, GL_VERTEX_ARRAY, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	// Needed states: GL_VERTEX_ARRAY, 
	// Unneeded states: GL_TEXTURE_2D, GL_TEXTURE_COORD_ARRAY, GL_COLOR_ARRAY	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	//glDisableClientState(GL_COLOR_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, (numofpoint+1)*2);
	
	// restore default state
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);	
	
	free( vertices );
	free( colors );
	
}

-(void)draw{
	if(whetherTouch == YES){
		percent+=0.03;
	}
	else 
		percent-=0.03;
	
	if(percent>1){
		percent=1;
		[[CCDirector sharedDirector] replaceScene:mscene];
		return;
	}
	if(percent<0)
		percent=0;
	
	if(percent==0)
		scolor = CGRectMake(CCRANDOM_0_1()*0.8+0.2,CCRANDOM_0_1()*0.8+0.2,CCRANDOM_0_1()*0.8+0.2,0);
	
	CGPoint p = ccp(screenSize.width/2,screenSize.height/2);
	float r = screenSize.height*3/7;
	CGRect c = CGRectMultiple(scolor, percent*1.5);
	
	startHex(p, 0, r, c, BlackWithAlpha(0), 6);
	startCircle(p, r*0.95, r, BlackWithAlpha(0), BlackWithAlpha(0), scolor,scolor, 64, 0, percent*360);
	startCircle(p, r, r*1.05,scolor,scolor, BlackWithAlpha(0), BlackWithAlpha(0), 64, 0, percent*360);
}

-(void)dealloc{
	[super dealloc];
	[mscene release];
}

-(id)init{
	if((self=[super init])){
		mscene = [[MainMenu scene] retain];
		self.isTouchEnabled = YES;
		screenSize = [[CCDirector sharedDirector] winSize];
		whetherTouch = NO;
		percent = 0;
		scolor = CGRectMake(1,1,1,1);
		
		CCLabelTTF *touch = [CCLabelTTF labelWithString:@"Touch to start" fontName:@"Marker Felt" fontSize:screenSize.height/10];
		touch.position = ccp(screenSize.width/2,screenSize.height/2);
		id action1 = [CCFadeOut actionWithDuration:2.0];
		id action2 = [CCFadeIn actionWithDuration:2.0];
		id actseq = [CCSequence actionOne:action1 two:action2];
		id repeat = [CCRepeatForever actionWithAction:actseq];
		[self addChild:touch];
		[touch runAction:repeat];
	}
	return self;
}

-(void)ccTouchesBegan:(NSSet*)touches withEvent:(UIEvent*)event{
	whetherTouch = YES;
}

-(void)ccTouchesMoved:(NSSet*)touches withEvent:(UIEvent*)event
{
	whetherTouch = YES;
}

-(void)ccTouchesEnded:(NSSet*)touches withEvent:(UIEvent*)event{
	whetherTouch = NO;
}

-(void)ccTouchesCancelled:(NSSet*)touches withEvent:(UIEvent*)event{
	whetherTouch = NO;
}



@end
